@@ -22,7 +22,9 @@ then
 fi
 export LIBTOOLIZE
 
-autoreconf -fi -I m4
+./update-version-h.sh
+
+autoreconf -fi
 
 if test "$GETTEXTIZE"; then
   echo "Creating aclocal.m4 ..."
