@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-core.c 10853 2010-06-25 17:49:53Z charles $
+ * $Id: tr-core.c 11113 2010-08-04 17:35:03Z Longinus00 $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -293,7 +293,7 @@ compareByName( GtkTreeModel *             model,
 
     gtk_tree_model_get( model, a, MC_NAME_COLLATED, &ca, -1 );
     gtk_tree_model_get( model, b, MC_NAME_COLLATED, &cb, -1 );
-    ret = strcmp( ca, cb );
+    ret = gtr_strcmp0( ca, cb );
     g_free( cb );
     g_free( ca );
     return ret;
@@ -472,14 +472,29 @@ tr_core_apply_defaults( tr_ctor * ctor )
     }
 }
 
-static int
-tr_strcmp( const void * a,
-           const void * b )
+static char *
+torrentTrackerString( tr_torrent * tor )
 {
-    if( a && b ) return strcmp( a, b );
-    if( a ) return 1;
-    if( b ) return -1;
-    return 0;
+    int i;
+    GString * str = g_string_new( NULL );
+    const tr_info * inf = tr_torrentInfo( tor );
+
+    for( i = 0; i < inf->trackerCount; ++i )
+    {
+        const tr_tracker_info * t = &inf->trackers[i];
+        g_string_append( str, t->announce );
+    }
+
+    return g_string_free( str, FALSE );
+}
+
+static gboolean
+isTorrentActive( tr_torrent * tor )
+{
+    const tr_stat * st = tr_torrentStat( tor );
+    return ( st->peersSendingToUs > 0 )
+        || ( st->peersGettingFromUs > 0 )
+        || ( st->activity == TR_STATUS_CHECK );
 }
 
 #ifdef HAVE_GIO
@@ -632,7 +647,7 @@ updateWatchDir( TrCore * core )
         PREF_KEY_DIR_WATCH_ENABLED );
     struct TrCorePrivate * p = TR_CORE( core )->priv;
 
-    if( p->monitor && ( !isEnabled || tr_strcmp( filename, p->monitor_path ) ) )
+    if( p->monitor && ( !isEnabled || gtr_strcmp0( filename, p->monitor_path ) ) )
     {
         g_signal_handler_disconnect( p->monitor, p->monitor_tag );
         g_free( p->monitor_path );
@@ -709,7 +724,11 @@ tr_core_init( GTypeInstance *  instance,
                       G_TYPE_POINTER,   /* tr_torrent* */
                       G_TYPE_DOUBLE,    /* tr_stat.pieceUploadSpeed */
                       G_TYPE_DOUBLE,    /* tr_stat.pieceDownloadSpeed */
-                      G_TYPE_INT };     /* tr_stat.status */
+                      G_TYPE_BOOLEAN,   /* filter.c:ACTIVITY_FILTER_ACTIVE */
+                      G_TYPE_INT,       /* tr_stat.activity */
+                      G_TYPE_UCHAR,     /* tr_stat.finished */
+                      G_TYPE_CHAR,      /* tr_priority_t */
+                      G_TYPE_STRING };  /* concatenated trackers string */
 
     p = self->priv = G_TYPE_INSTANCE_GET_PRIVATE( self,
                                                   TR_CORE_TYPE,
@@ -843,6 +862,7 @@ tr_core_add_torrent( TrCore     * self,
     const tr_stat * st = tr_torrent_stat( gtor );
     tr_torrent *    tor = tr_torrent_handle( gtor );
     char *          collated = doCollate( inf->name );
+    char *          trackers = torrentTrackerString( tor );
     GtkListStore *  store = GTK_LIST_STORE( tr_core_model( self ) );
     GtkTreeIter     unused;
 
@@ -853,7 +873,11 @@ tr_core_add_torrent( TrCore     * self,
                                        MC_TORRENT_RAW,   tor,
                                        MC_SPEED_UP,      st->pieceUploadSpeed,
                                        MC_SPEED_DOWN,    st->pieceDownloadSpeed,
+                                       MC_ACTIVE,        isTorrentActive( tor ),
                                        MC_ACTIVITY,      st->activity,
+                                       MC_FINISHED,      st->finished,
+                                       MC_PRIORITY,      tr_torrentGetPriority( tor ),
+                                       MC_TRACKERS,      trackers,
                                        -1 );
 
     if( doNotify )
@@ -862,6 +886,7 @@ tr_core_add_torrent( TrCore     * self,
     /* cleanup */
     g_object_unref( G_OBJECT( gtor ) );
     g_free( collated );
+    g_free( trackers );
 }
 
 int
@@ -1242,33 +1267,55 @@ update_foreach( GtkTreeModel * model,
                 gpointer       data UNUSED )
 {
     int oldActivity, newActivity;
+    tr_bool oldFinished, newFinished;
+    tr_priority_t oldPriority, newPriority;
+    char * oldTrackers, * newTrackers;
     double oldUpSpeed, newUpSpeed;
     double oldDownSpeed, newDownSpeed;
+    gboolean oldActive, newActive;
     const tr_stat * st;
     TrTorrent * gtor;
+    tr_torrent * tor;
 
     /* get the old states */
     gtk_tree_model_get( model, iter,
                         MC_TORRENT, &gtor,
+                        MC_ACTIVE, &oldActive,
                         MC_ACTIVITY, &oldActivity,
+                        MC_FINISHED, &oldFinished,
+                        MC_PRIORITY, &oldPriority,
+                        MC_TRACKERS, &oldTrackers,
                         MC_SPEED_UP, &oldUpSpeed,
                         MC_SPEED_DOWN, &oldDownSpeed,
                         -1 );
 
     /* get the new states */
-    st = tr_torrentStat( tr_torrent_handle( gtor ) );
+    tor = tr_torrent_handle( gtor );
+    st = tr_torrentStat( tor );
+    newActive = isTorrentActive( tor );
     newActivity = st->activity;
+    newFinished = st->finished;
+    newPriority = tr_torrentGetPriority( tor );
+    newTrackers = torrentTrackerString( tor );
     newUpSpeed = st->pieceUploadSpeed;
     newDownSpeed = st->pieceDownloadSpeed;
 
     /* updating the model triggers off resort/refresh,
        so don't do it unless something's actually changed... */
-    if( ( newActivity != oldActivity ) ||
-        ( (int)(newUpSpeed*10.0) != (int)(oldUpSpeed*10.0) ) ||
-        ( (int)(newDownSpeed*10.0) != (int)(oldDownSpeed*10.0) ) )
+    if( ( newActive != oldActive )
+        || ( newActivity  != oldActivity )
+        || ( newFinished != oldFinished )
+        || ( newPriority != oldPriority )
+        || ( gtr_strcmp0( oldTrackers, newTrackers ) )
+        || ( (int)(newUpSpeed*10.0) != (int)(oldUpSpeed*10.0) )
+        || ( (int)(newDownSpeed*10.0) != (int)(oldDownSpeed*10.0) ) )
     {
         gtk_list_store_set( GTK_LIST_STORE( model ), iter,
+                            MC_ACTIVE, newActive,
                             MC_ACTIVITY, newActivity,
+                            MC_FINISHED, newFinished,
+                            MC_PRIORITY, newPriority,
+                            MC_TRACKERS, newTrackers,
                             MC_SPEED_UP, newUpSpeed,
                             MC_SPEED_DOWN, newDownSpeed,
                             -1 );
@@ -1276,6 +1323,8 @@ update_foreach( GtkTreeModel * model,
 
     /* cleanup */
     g_object_unref( gtor );
+    g_free( newTrackers );
+    g_free( oldTrackers );
     return FALSE;
 }
 
@@ -1456,7 +1505,7 @@ tr_core_set_pref( TrCore *     self,
 {
     const char * oldval = pref_string_get( key );
 
-    if( tr_strcmp( oldval, newval ) )
+    if( gtr_strcmp0( oldval, newval ) )
     {
         pref_string_set( key, newval );
         commitPrefsChange( self, key );
