@@ -1,21 +1,11 @@
 static const char * fallback_ui_file =
     "<ui>\n"
     "  <menubar name='main-window-menu'>\n"
-    "    <menu action='torrent-menu'>\n"
+    "    <menu action='file-menu'>\n"
     "      <menuitem action='add-torrent-menu'/>\n"
+    "      <menuitem action='add-torrent-from-url'/>\n"
     "      <menuitem action='new-torrent'/>\n"
     "      <separator/>\n"
-    "      <menuitem action='show-torrent-properties'/>\n"
-    "      <menuitem action='open-torrent-folder'/>\n"
-    "      <menuitem action='relocate-torrent'/>\n"
-    "      <separator/>\n"
-    "      <menuitem action='start-torrent'/>\n"
-    "      <menuitem action='update-tracker'/>\n"
-    "      <menuitem action='pause-torrent'/>\n"
-    "      <menuitem action='verify-torrent'/>\n"
-    "      <menuitem action='remove-torrent'/>\n"
-    "      <menuitem action='delete-torrent'/>\n"
-    "      <separator/>\n"
     "      <menuitem action='start-all-torrents'/>\n"
     "      <menuitem action='pause-all-torrents'/>\n"
     "      <separator/>\n"
@@ -27,8 +17,23 @@ static const char * fallback_ui_file =
     "      <separator/>\n"
     "      <menuitem action='edit-preferences'/>\n"
     "    </menu>\n"
+    "    <menu action='torrent-menu'>\n"
+    "      <menuitem action='show-torrent-properties'/>\n"
+    "      <menuitem action='open-torrent-folder'/>\n"
+    "      <separator/>\n"
+    "      <menuitem action='start-torrent'/>\n"
+    "      <menuitem action='update-tracker'/>\n"
+    "      <menuitem action='pause-torrent'/>\n"
+    "      <menuitem action='copy-magnet-link-to-clipboard'/>\n"
+    "      <separator/>\n"
+    "      <menuitem action='relocate-torrent'/>\n"
+    "      <menuitem action='verify-torrent'/>\n"
+    "      <separator/>\n"
+    "      <menuitem action='remove-torrent'/>\n"
+    "      <menuitem action='delete-torrent'/>\n"
+    "    </menu>\n"
     "    <menu action='view-menu'>\n"
-    "      <menuitem action='minimal-view'/>\n"
+    "      <menuitem action='compact-view'/>\n"
     "      <separator/>\n"
     "      <menuitem action='show-toolbar'/>\n"
     "      <menuitem action='show-filterbar'/>\n"
@@ -36,12 +41,12 @@ static const char * fallback_ui_file =
     "      <separator/>\n"
     "      <menuitem action='sort-by-activity'/>\n"
     "      <menuitem action='sort-by-age'/>\n"
-    "      <menuitem action='sort-by-eta'/>\n"
     "      <menuitem action='sort-by-name'/>\n"
     "      <menuitem action='sort-by-progress'/>\n"
     "      <menuitem action='sort-by-ratio'/>\n"
     "      <menuitem action='sort-by-size'/>\n"
     "      <menuitem action='sort-by-state'/>\n"
+    "      <menuitem action='sort-by-time-left'/>\n"
     "      <menuitem action='sort-by-tracker'/>\n"
     "      <separator/>\n"
     "      <menuitem action='sort-reversed'/>\n"
@@ -50,6 +55,8 @@ static const char * fallback_ui_file =
     "      <menuitem action='toggle-message-log'/>\n"
     "      <menuitem action='show-stats'/>\n"
     "      <separator/>\n"
+    "      <menuitem action='donate'/>\n"
+    "      <separator/>\n"
     "      <menuitem action='help'/>\n"
     "      <menuitem action='show-about-dialog'/>\n"
     "    </menu>\n"
@@ -67,28 +74,44 @@ static const char * fallback_ui_file =
     "  <popup name='main-window-popup'>\n"
     "    <menuitem action='show-torrent-properties'/>\n"
     "    <menuitem action='open-torrent-folder'/>\n"
-    "    <menuitem action='relocate-torrent'/>\n"
+    "    <separator/>\n"
+    "    <menu action='sort-menu'>\n"
+    "      <menuitem action='sort-by-activity'/>\n"
+    "      <menuitem action='sort-by-age'/>\n"
+    "      <menuitem action='sort-by-name'/>\n"
+    "      <menuitem action='sort-by-progress'/>\n"
+    "      <menuitem action='sort-by-ratio'/>\n"
+    "      <menuitem action='sort-by-size'/>\n"
+    "      <menuitem action='sort-by-state'/>\n"
+    "      <menuitem action='sort-by-time-left'/>\n"
+    "      <menuitem action='sort-by-tracker'/>\n"
+    "      <separator/>\n"
+    "      <menuitem action='sort-reversed'/>\n"
+    "    </menu>\n"
     "    <separator/>\n"
     "    <menuitem action='start-torrent'/>\n"
-    "    <menuitem action='pause-torrent'/>\n"
-    "    <menuitem action='verify-torrent'/>\n"
     "    <menuitem action='update-tracker'/>\n"
+    "    <menuitem action='pause-torrent'/>\n"
+    "    <menuitem action='copy-magnet-link-to-clipboard'/>\n"
+    "    <separator/>\n"
+    "    <menuitem action='verify-torrent'/>\n"
+    "    <menuitem action='relocate-torrent'/>\n"
     "    <separator/>\n"
     "    <menuitem action='remove-torrent'/>\n"
     "    <menuitem action='delete-torrent'/>\n"
     "  </popup>\n"
     "\n"
     "  <popup name='icon-popup'>\n"
-    "    <menuitem action='add-torrent-menu'/>\n"
-    "    <separator/>\n"
     "    <menuitem action='toggle-main-window'/>\n"
-    "    <menuitem action='toggle-message-log'/>\n"
+    "    <separator/>\n"
+    "    <menuitem action='add-torrent-menu'/>\n"
+    "    <menuitem action='add-torrent-from-url'/>\n"
+    "    <separator/>\n"
+    "    <menuitem action='pause-all-torrents'/>\n"
+    "    <menuitem action='start-all-torrents'/>\n"
+    "    <separator/>\n"
     "    <menuitem action='alt-speed-enabled'/>\n"
     "    <separator/>\n"
-    "    <menuitem action='start-all-torrents'/>\n"
-    "    <menuitem action='pause-all-torrents'/>\n"
-    "    <separator/>\n"
-    "    <menuitem action='show-about-dialog'/>\n"
     "    <menuitem action='quit'/>\n"
     "  </popup>\n"
     "\n"
