@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: util.c 10827 2010-06-23 16:31:09Z charles $
+ * $Id: util.c 11110 2010-08-04 14:11:06Z charles $
  */
 
 #include <ctype.h> /* isxdigit() */
@@ -457,6 +457,19 @@ gtr_object_ref_sink( gpointer object )
     return object;
 }
 
+int
+gtr_strcmp0( const char * str1, const char * str2 )
+{
+#if GLIB_CHECK_VERSION( 2, 16, 0 )
+    return g_strcmp0( str1, str2 );
+#else
+    if( str1 && str2 ) return strcmp( str1, str2 );
+    if( str1 ) return 1;
+    if( str2 ) return -1;
+    return 0;
+#endif
+}
+
 int
 gtr_file_trash_or_remove( const char * filename )
 {
