@@ -7,13 +7,12 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: bencode.c 10584 2010-05-01 04:40:53Z charles $
+ * $Id: bencode.c 10634 2010-05-06 17:02:31Z charles $
  */
 
 #include <assert.h>
 #include <ctype.h> /* isdigit() */
 #include <errno.h>
-#include <limits.h> /* PATH_MAX */
 #include <math.h> /* fabs() */
 #include <stdio.h>
 #include <stdlib.h> /* realpath() */
@@ -32,6 +31,7 @@
 #include "bencode.h"
 #include "json.h"
 #include "list.h"
+#include "platform.h" /* TR_PATH_MAX */
 #include "ptrarray.h"
 #include "utils.h" /* tr_new(), tr_free() */
 
@@ -1621,7 +1621,7 @@ tr_bencToFile( const tr_benc * top, tr_fmt_mode mode, const char * filename )
     char * tmp;
     int fd;
     int err = 0;
-    char buf[PATH_MAX];
+    char buf[TR_PATH_MAX];
 
     /* follow symlinks to find the "real" file, to make sure the temporary
      * we build with mkstemp() is created on the right partition */
