@@ -1,26 +1,14 @@
-/******************************************************************************
- * $Id: fdlimit.h 9931 2010-01-14 14:20:49Z charles $
+/*
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
- * Copyright (c) 2005-2010 Transmission authors and contributors
+ * This file is licensed by the GPL version 2.  Works owned by the
+ * Transmission project are granted a special exemption to clause 2(b)
+ * so that the bulk of its code can remain under the MIT license.
+ * This exemption does not extend to derived works not owned by
+ * the Transmission project.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- *****************************************************************************/
+ * $Id: fdlimit.h 10905 2010-06-29 01:21:38Z charles $
+ */
 
 #ifndef __TRANSMISSION__
  #error only libtransmission should #include this header.
@@ -50,11 +38,8 @@ int tr_open_file_for_writing( const char * filename );
 
 void tr_close_file( int fd );
 
-tr_bool tr_preallocate_file( const char * filename, uint64_t length );
-
-int64_t tr_lseek( int fd, int64_t offset, int whence );
 ssize_t tr_pread(int fd, void *buf, size_t count, off_t offset);
-ssize_t tr_pwrite(int fd, void *buf, size_t count, off_t offset);
+ssize_t tr_pwrite(int fd, const void *buf, size_t count, off_t offset);
 int tr_prefetch(int fd, off_t offset, size_t count);
 
 
