@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ratecontrol.h 7664 2009-01-11 17:46:51Z charles $
+ * $Id: ratecontrol.h 9847 2010-01-01 22:26:35Z charles $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -63,9 +63,9 @@ tr_ratecontrol;
 ****
 ***/
 
-static TR_INLINE void tr_rcConstruct ( tr_ratecontrol * rc ) { memset( rc, 0, sizeof( tr_ratecontrol ) ); }
+static inline void tr_rcConstruct ( tr_ratecontrol * rc ) { memset( rc, 0, sizeof( tr_ratecontrol ) ); }
 
-static TR_INLINE void tr_rcDestruct  ( tr_ratecontrol * rc ) { memset( rc, 0xDEAD, sizeof( tr_ratecontrol ) ); }
+static inline void tr_rcDestruct  ( tr_ratecontrol * rc ) { memset( rc, 0xDEAD, sizeof( tr_ratecontrol ) ); }
 
 void             tr_rcTransferred ( tr_ratecontrol         * ratecontrol,
                                     size_t                   byteCount );
