@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: rpc-server.c 10985 2010-07-09 15:22:23Z charles $
+ * $Id: rpc-server.c 11111 2010-08-04 14:21:35Z charles $
  */
 
 #include <assert.h>
@@ -608,8 +608,7 @@ handle_request( struct evhttp_request * req, void * arg )
                || !strcmp( req->uri, "/transmission/clutch" )
                || !strcmp( req->uri, "/" ) )
         {
-            const char * location = "transmission/web/";
-            evhttp_add_header( req->output_headers, "Location", location );
+            evhttp_add_header( req->output_headers, "Location", "/transmission/web/" );
             send_simple_response( req, HTTP_MOVEPERM, NULL );
         }
         else if( !strncmp( req->uri, "/transmission/web/", 18 ) )
