@@ -95,8 +95,54 @@ tr_torrentSetMetadataSizeHint( tr_torrent * tor, int size )
     }
 }
 
+static int
+findInfoDictOffset( const tr_torrent * tor )
+{
+    size_t fileLen;
+    uint8_t * fileContents;
+    int offset = 0;
+
+    /* load the file, and find the info dict's offset inside the file */
+    if(( fileContents = tr_loadFile( tor->info.torrent, &fileLen )))
+    {
+        tr_benc top;
+
+        if( !tr_bencParse( fileContents, fileContents + fileLen, &top, NULL ) )
+        {
+            tr_benc * infoDict;
+
+            if( tr_bencDictFindDict( &top, "info", &infoDict ) )
+            {
+                int infoLen;
+                char * infoContents = tr_bencToStr( infoDict, TR_FMT_BENC, &infoLen );
+                const uint8_t * i = (const uint8_t*) tr_memmem( (char*)fileContents, fileLen, infoContents, infoLen );
+                offset = i == NULL ? i - fileContents : 0;
+                tr_free( infoContents );
+            }
+
+            tr_bencFree( &top );
+        }
+
+        tr_free( fileContents );
+    }
+
+    return offset;
+}
+
+static void
+ensureInfoDictOffsetIsCached( tr_torrent * tor )
+{
+    assert( tr_torrentHasMetadata( tor ) );
+
+    if( !tor->infoDictOffsetIsCached )
+    {
+        tor->infoDictOffset = findInfoDictOffset( tor );
+        tor->infoDictOffsetIsCached = TRUE;
+    }
+}
+
 void*
-tr_torrentGetMetadataPiece( const tr_torrent * tor, int piece, int * len )
+tr_torrentGetMetadataPiece( tr_torrent * tor, int piece, int * len )
 {
     char * ret = NULL;
 
@@ -104,9 +150,16 @@ tr_torrentGetMetadataPiece( const tr_torrent * tor, int piece, int * len )
     assert( piece >= 0 );
     assert( len != NULL );
 
-    if( tor->infoDictLength > 0 )
+    if( tr_torrentHasMetadata( tor ) )
     {
-        FILE * fp = fopen( tor->info.torrent, "rb" );
+        FILE * fp;
+
+        ensureInfoDictOffsetIsCached( tor );
+
+        assert( tor->infoDictLength > 0 );
+        assert( tor->infoDictOffset >= 0 );
+
+        fp = fopen( tor->info.torrent, "rb" );
         if( fp != NULL )
         {
             const int o = piece  * METADATA_PIECE_SIZE;
@@ -200,6 +253,8 @@ tr_torrentSetMetadataPiece( tr_torrent  * tor, int piece, const void  * data, in
                 if( !tr_bencLoadFile( &newMetainfo, TR_FMT_BENC, path ) )
                 {
                     tr_bool hasInfo;
+                    tr_info info;
+                    int infoDictLength;
 
                     /* remove any old .torrent and .resume files */
                     remove( path );
@@ -208,21 +263,31 @@ tr_torrentSetMetadataPiece( tr_torrent  * tor, int piece, const void  * data, in
                     dbgmsg( tor, "Saving completed metadata to \"%s\"", path );
                     tr_bencMergeDicts( tr_bencDictAddDict( &newMetainfo, "info", 0 ), &infoDict );
 
-                    success = tr_metainfoParse( tor->session, &newMetainfo, &tor->info,
-                                                &hasInfo, &tor->infoDictOffset, &tor->infoDictLength );
+                    memset( &info, 0, sizeof( tr_info ) );
+                    success = tr_metainfoParse( tor->session, &newMetainfo, &info, &hasInfo, &infoDictLength );
 
-                    assert( hasInfo );
-                    assert( success );
+                    if( success && !tr_getBlockSize( info.pieceSize ) )
+                    {
+                        tr_torrentSetLocalError( tor, _( "Magnet torrent's metadata is not usable" ) );
+                        success = FALSE;
+                    }
 
-                    /* save the new .torrent file */
-                    tr_bencToFile( &newMetainfo, TR_FMT_BENC, tor->info.torrent );
-                    tr_sessionSetTorrentFile( tor->session, tor->info.hashString, tor->info.torrent );
-                    tr_torrentGotNewInfoDict( tor );
-                    tr_torrentSetDirty( tor );
+                    if( success )
+                    {
+                        /* keep the new info */
+                        tor->info = info;
+                        tor->infoDictLength = infoDictLength;
+
+                        /* save the new .torrent file */
+                        tr_bencToFile( &newMetainfo, TR_FMT_BENC, tor->info.torrent );
+                        tr_sessionSetTorrentFile( tor->session, tor->info.hashString, tor->info.torrent );
+                        tr_torrentGotNewInfoDict( tor );
+                        tr_torrentSetDirty( tor );
+                    }
 
                     tr_bencFree( &newMetainfo );
                 }
-        
+
                 tr_bencFree( &infoDict );
                 tr_free( path );
             }
@@ -306,14 +371,19 @@ tr_torrentGetMagnetLink( const tr_torrent * tor )
 {
     int i;
     char * ret;
+    const char * name;
     struct evbuffer * s;
 
     assert( tr_isTorrent( tor ) );
 
     s = evbuffer_new( );
     evbuffer_add_printf( s, "magnet:?xt=urn:btih:%s", tor->info.hashString );
-    evbuffer_add_printf( s, "%s", "&dn=" );
-    tr_http_escape( s, tr_torrentName( tor ), -1, TRUE );
+    name = tr_torrentName( tor );
+    if( name && *name )
+    {
+        evbuffer_add_printf( s, "%s", "&dn=" );
+        tr_http_escape( s, tr_torrentName( tor ), -1, TRUE );
+    }
     for( i=0; i<tor->info.trackerCount; ++i )
     {
         evbuffer_add_printf( s, "%s", "&tr=" );
