@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent.h 10084 2010-02-02 22:45:22Z charles $
+ * $Id: torrent.h 10878 2010-06-26 18:52:05Z livings124 $
  */
 
 #ifndef __TRANSMISSION__
@@ -43,10 +43,10 @@ void        tr_ctorInitTorrentWanted( const tr_ctor * ctor, tr_torrent * tor );
 **/
 
 /* just like tr_torrentSetFileDLs but doesn't trigger a fastresume save */
-void        tr_torrentInitFileDLs( tr_torrent *      tor,
-                                   tr_file_index_t * files,
-                                   tr_file_index_t   fileCount,
-                                   tr_bool           do_download );
+void        tr_torrentInitFileDLs( tr_torrent              * tor,
+                                   const tr_file_index_t   * files,
+                                   tr_file_index_t          fileCount,
+                                   tr_bool                  do_download );
 
 void        tr_torrentRecheckCompleteness( tr_torrent * );
 
@@ -101,9 +101,6 @@ void             tr_torrentSetFileChecked( tr_torrent       * tor,
 
 void             tr_torrentUncheck( tr_torrent * tor );
 
-int              tr_torrentPromoteTracker( tr_torrent   * tor,
-                                           int            trackerIndex );
-
 time_t*          tr_torrentGetMTimes( const tr_torrent  * tor,
                                       size_t            * setmeCount );
 
@@ -130,6 +127,8 @@ tr_verify_state;
 void             tr_torrentSetVerifyState( tr_torrent      * tor,
                                            tr_verify_state   state );
 
+tr_torrent_activity tr_torrentGetActivity( tr_torrent * tor );
+
 struct tr_incomplete_metadata;
 
 /** @brief Torrent object */
@@ -142,6 +141,7 @@ struct tr_torrent
 
     tr_stat_errtype          error;
     char                     errorString[128];
+    char                     errorTracker[128];
 
     uint8_t                  obfuscatedHash[SHA_DIGEST_LENGTH];
 
@@ -165,10 +165,13 @@ struct tr_torrent
     /* Where the files are when the torrent is incomplete */
     char * incompleteDir;
 
-    /* Length, in bytes, of the "info" dict in the .torrent file */
+    /* Length, in bytes, of the "info" dict in the .torrent file. */
     int infoDictLength;
 
-    /* Offset, in bytes, of the beginning of the "info" dict in the .torrent file */
+    /* Offset, in bytes, of the beginning of the "info" dict in the .torrent file.
+     *
+     * Used by the torrent-magnet code for serving metainfo to peers.
+     * This field is lazy-generated and might not be initialized yet. */
     int infoDictOffset;
 
     /* Where the files are now.
@@ -197,6 +200,8 @@ struct tr_torrent
     time_t                     dhtAnnounce6At;
     tr_bool                    dhtAnnounceInProgress;
     tr_bool                    dhtAnnounce6InProgress;
+    
+    time_t                     lpdAnnounceAt;
 
     uint64_t                   downloadedCur;
     uint64_t                   downloadedPrev;
@@ -226,11 +231,13 @@ struct tr_torrent
     void                             * ratio_limit_hit_func_user_data;
 
     tr_bool                    isRunning;
+    tr_bool                    isStopping;
     tr_bool                    isDeleting;
-    tr_bool                    needsSeedRatioCheck;
     tr_bool                    startAfterVerify;
     tr_bool                    isDirty;
 
+    tr_bool                    infoDictOffsetIsCached;
+
     uint16_t                   maxConnectedPeers;
 
     tr_verify_state            verifyState;
@@ -331,6 +338,13 @@ static inline tr_bool tr_torrentAllowsDHT( const tr_torrent * tor )
         && ( !tr_torrentIsPrivate( tor ) );
 }
 
+static inline tr_bool tr_torrentAllowsLPD( const tr_torrent * tor )
+{
+    return ( tor != NULL )
+        && ( tr_sessionAllowsLPD( tor->session ) )
+        && ( !tr_torrentIsPrivate( tor ) );
+}
+
 static inline tr_bool tr_torrentIsPieceChecked( const tr_torrent  * tor,
                                                    tr_piece_index_t    i )
 {
@@ -371,6 +385,8 @@ const char * tr_torrentName( const tr_torrent * tor )
     return tor->info.name;
 }
 
+uint32_t tr_getBlockSize( uint32_t pieceSize );
+
 /**
  * Tell the tr_torrent that one of its files has become complete
  */
