@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: utils.c 10964 2010-07-07 16:48:24Z charles $
+ * $Id: utils.c 11050 2010-07-24 17:09:41Z charles $
  */
 
 #ifdef HAVE_MEMMEM
@@ -1404,7 +1404,7 @@ double
 tr_truncd( double x, int decimal_places )
 {
     const int i = (int) pow( 10, decimal_places );  
-    double x2 = (int)(x*i);
+    const double x2 = (int64_t)(x * i);
     return x2 / i;
 }
 
