@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: utils.h 10274 2010-02-23 06:06:45Z charles $
+ * $Id: utils.h 10964 2010-07-07 16:48:24Z charles $
  */
 
 #ifndef TR_UTILS_H
@@ -20,8 +20,6 @@
 #include <stdlib.h> /* malloc() */
 #include <time.h> /* time_t */
 
-#include "transmission.h"
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -267,16 +265,15 @@ void tr_timerAddMsec( struct event * timer, int milliseconds ) TR_GNUC_NONNULL(1
 uint64_t tr_date( void );
 
 /** @brief sleep the specified number of milliseconds */
-void tr_wait_msec( uint64_t delay_milliseconds );
+void tr_wait_msec( long int delay_milliseconds );
 
 /**
  * @brief make a copy of 'str' whose non-utf8 content has been corrected or stripped
  * @return a newly-allocated string that must be freed with tr_free()
  * @param str the string to make a clean copy of
  * @param len the length of the string to copy.  If -1, the entire string is used.
- * @param err if an error occurs and err is non-NULL, it's set to TRUE.
  */
-char* tr_utf8clean( const char * str, int len, tr_bool * err ) TR_GNUC_MALLOC;
+char* tr_utf8clean( const char * str, int len ) TR_GNUC_MALLOC;
 
 
 /***
@@ -402,6 +399,10 @@ const char* tr_strerror( int );
     @return the stripped string */
 char* tr_strstrip( char * str );
 
+/** @brief Returns true if the string ends with the specified case-insensitive suffix */
+tr_bool tr_str_has_suffix( const char *str, const char *suffix );
+
+
 /** @brief Portability wrapper for memmem() that uses the system implementation if available */
 const char* tr_memmem( const char * haystack, size_t haystack_len,
                        const char * needle, size_t needle_len );
@@ -541,8 +542,8 @@ extern time_t transmission_now;
  */
 static inline time_t tr_time( void ) { return transmission_now; }
 
-/** @brief Private libtransmission function to update the second counter used by tr_time() */
-void tr_timeUpdate( time_t now );
+/** @brief Private libtransmission function to update tr_time()'s counter */
+static inline void tr_timeUpdate( time_t now ) { transmission_now = now; }
 
 /***
 ****
