@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: Torrent.h 10212 2010-02-15 19:26:23Z livings124 $
+ * $Id: Torrent.h 10610 2010-05-01 16:19:40Z livings124 $
  *
  * Copyright (c) 2006-2010 Transmission authors and contributors
  *
@@ -47,7 +47,7 @@
     NSIndexSet * fPreviousFinishedIndexes;
     NSDate * fPreviousFinishedIndexesDate;
     
-    BOOL fFinishedSeeding, fWaitToStart, fStalled;
+    BOOL fWaitToStart, fStalled;
     
     NSInteger fGroupValue;
     
@@ -161,6 +161,7 @@
 - (BOOL) isCheckingWaiting;
 - (BOOL) allDownloaded;
 - (BOOL) isComplete;
+- (BOOL) isFinishedSeeding;
 - (BOOL) isError;
 - (BOOL) isAnyErrorOrWarning;
 - (NSString *) errorMessage;
@@ -182,6 +183,7 @@
 - (NSInteger) totalPeersCache;
 - (NSInteger) totalPeersPex;
 - (NSInteger) totalPeersDHT;
+- (NSInteger) totalPeersLocal;
 - (NSInteger) totalPeersLTEP;
 - (NSInteger) totalPeersKnown;
 
