@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: TorrentTableView.m 10206 2010-02-15 14:56:14Z livings124 $
+ * $Id: TorrentTableView.m 10407 2010-03-20 18:54:24Z livings124 $
  *
  * Copyright (c) 2005-2010 Transmission authors and contributors
  *
@@ -66,6 +66,7 @@
         else
             fCollapsedGroups = [[NSMutableIndexSet alloc] init];
         
+        fMouseRow = -1;
         fMouseControlRow = -1;
         fMouseRevealRow = -1;
         fMouseActionRow = -1;
@@ -152,6 +153,7 @@
         [cell setRepresentedObject: item];
         
         const NSInteger row = [self rowForItem: item];
+        [cell setHover: row == fMouseRow];
         [cell setControlHover: row == fMouseControlRow];
         [cell setRevealHover: row == fMouseRevealRow];
         [cell setActionHover: row == fMouseActionRow];
@@ -226,9 +228,9 @@
 - (void) updateTrackingAreas
 {
     [super updateTrackingAreas];
-    [self removeButtonTrackingAreas];
+    [self removeTrackingAreas];
     
-    NSRange rows = [self rowsInRect: [self visibleRect]];
+    const NSRange rows = [self rowsInRect: [self visibleRect]];
     if (rows.length == 0)
         return;
     
@@ -244,8 +246,9 @@
     }
 }
 
-- (void) removeButtonTrackingAreas
+- (void) removeTrackingAreas
 {
+    fMouseRow = -1;
     fMouseControlRow = -1;
     fMouseRevealRow = -1;
     fMouseActionRow = -1;
@@ -257,6 +260,15 @@
     }
 }
 
+- (void) setRowHover: (NSInteger) row
+{
+    NSAssert([fDefaults boolForKey: @"SmallView"], @"cannot set a hover row when not in compact view");
+    
+    fMouseRow = row;
+    if (row >= 0)
+        [self setNeedsDisplayInRect: [self rectOfRow: row]];
+}
+
 - (void) setControlButtonHover: (NSInteger) row
 {
     fMouseControlRow = row;
@@ -291,8 +303,14 @@
             fMouseActionRow = rowVal;
         else if ([type isEqualToString: @"Control"])
             fMouseControlRow = rowVal;
-        else
+        else if ([type isEqualToString: @"Reveal"])
             fMouseRevealRow = rowVal;
+        else
+        {
+            fMouseRow = rowVal;
+            if (![fDefaults boolForKey: @"SmallView"])
+                return;
+        }
         
         [self setNeedsDisplayInRect: [self rectOfRow: rowVal]];
     }
@@ -310,8 +328,14 @@
             fMouseActionRow = -1;
         else if ([type isEqualToString: @"Control"])
             fMouseControlRow = -1;
-        else
+        else if ([type isEqualToString: @"Reveal"])
             fMouseRevealRow = -1;
+        else
+        {
+            fMouseRow = -1;
+            if (![fDefaults boolForKey: @"SmallView"])
+                return;
+        }
         
         [self setNeedsDisplayInRect: [self rectOfRow: [row integerValue]]];
     }
@@ -660,71 +684,74 @@
 //alternating rows - first row after group row is white
 - (void) highlightSelectionInClipRect: (NSRect) clipRect
 {
-    NSRect visibleRect = clipRect;
-    NSRange rows = [self rowsInRect: visibleRect];
-    BOOL start = YES;
-    
-    const CGFloat totalRowHeight = [self rowHeight] + [self intercellSpacing].height;
-    
-    NSRect gridRects[(NSInteger)(ceil(visibleRect.size.height / totalRowHeight / 2.0)) + 1]; //add one if partial rows at top and bottom
-    NSInteger rectNum = 0;
-    
-    if (rows.length > 0)
+    if (![fDefaults boolForKey: @"SmallView"])
     {
-        //determine what the first row color should be
-        if ([[self itemAtRow: rows.location] isKindOfClass: [Torrent class]])
+        NSRect visibleRect = clipRect;
+        NSRange rows = [self rowsInRect: visibleRect];
+        BOOL start = YES;
+        
+        const CGFloat totalRowHeight = [self rowHeight] + [self intercellSpacing].height;
+        
+        NSRect gridRects[(NSInteger)(ceil(visibleRect.size.height / totalRowHeight / 2.0)) + 1]; //add one if partial rows at top and bottom
+        NSInteger rectNum = 0;
+        
+        if (rows.length > 0)
         {
-            for (NSInteger i = rows.location-1; i>=0; i--)
+            //determine what the first row color should be
+            if ([[self itemAtRow: rows.location] isKindOfClass: [Torrent class]])
+            {
+                for (NSInteger i = rows.location-1; i>=0; i--)
+                {
+                    if (![[self itemAtRow: i] isKindOfClass: [Torrent class]])
+                        break;
+                    start = !start;
+                }
+            }
+            else
+            {
+                rows.location++;
+                rows.length--;
+            }
+            
+            NSInteger i;
+            for (i = rows.location; i < NSMaxRange(rows); i++)
             {
                 if (![[self itemAtRow: i] isKindOfClass: [Torrent class]])
-                    break;
+                {
+                    start = YES;
+                    continue;
+                }
+                
+                if (!start && ![self isRowSelected: i])
+                    gridRects[rectNum++] = [self rectOfRow: i];
+                
                 start = !start;
             }
-        }
-        else
-        {
-            rows.location++;
-            rows.length--;
+            
+            const CGFloat newY = NSMaxY([self rectOfRow: i-1]);
+            visibleRect.size.height -= newY - visibleRect.origin.y;
+            visibleRect.origin.y = newY;
         }
         
-        NSInteger i;
-        for (i = rows.location; i < NSMaxRange(rows); i++)
+        const NSInteger numberBlankRows = ceil(visibleRect.size.height / totalRowHeight);
+        
+        //remaining visible rows continue alternating
+        visibleRect.size.height = totalRowHeight;
+        if (start)
+            visibleRect.origin.y += totalRowHeight;
+        
+        for (NSInteger i = start ? 1 : 0; i < numberBlankRows; i += 2)
         {
-            if (![[self itemAtRow: i] isKindOfClass: [Torrent class]])
-            {
-                start = YES;
-                continue;
-            }
-            
-            if (!start && ![self isRowSelected: i])
-                gridRects[rectNum++] = [self rectOfRow: i];
-            
-            start = !start;
+            gridRects[rectNum++] = visibleRect;
+            visibleRect.origin.y += 2.0 * totalRowHeight;
         }
         
-        const CGFloat newY = NSMaxY([self rectOfRow: i-1]);
-        visibleRect.size.height -= newY - visibleRect.origin.y;
-        visibleRect.origin.y = newY;
+        NSAssert([[NSColor controlAlternatingRowBackgroundColors] count] >= 2, @"There should be 2 alternating row colors");
+        
+        [[[NSColor controlAlternatingRowBackgroundColors] objectAtIndex: 1] set];
+        NSRectFillList(gridRects, rectNum);
     }
     
-    const NSInteger numberBlankRows = ceil(visibleRect.size.height / totalRowHeight);
-    
-    //remaining visible rows continue alternating
-    visibleRect.size.height = totalRowHeight;
-    if (start)
-        visibleRect.origin.y += totalRowHeight;
-    
-    for (NSInteger i = start ? 1 : 0; i < numberBlankRows; i += 2)
-    {
-        gridRects[rectNum++] = visibleRect;
-        visibleRect.origin.y += 2.0 * totalRowHeight;
-    }
-    
-    NSAssert([[NSColor controlAlternatingRowBackgroundColors] count] >= 2, @"There should be 2 alternating row colors");
-    
-    [[[NSColor controlAlternatingRowBackgroundColors] objectAtIndex: 1] set];
-    NSRectFillList(gridRects, rectNum);
-    
     [super highlightSelectionInClipRect: clipRect];
 }
 
