@@ -41,8 +41,8 @@ Torrent._MetaDataFields = [ 'addedDate', 'comment', 'creator', 'dateCreated',
 Torrent._DynamicFields = [ 'downloadedEver', 'error', 'errorString', 'eta',
     'haveUnchecked', 'haveValid', 'leftUntilDone', 'metadataPercentComplete', 'peersConnected',
     'peersGettingFromUs', 'peersSendingToUs', 'rateDownload', 'rateUpload',
-    'recheckProgress', 'sizeWhenDone', 'status', 'trackerStats',
-    'uploadedEver', 'uploadRatio', 'seedRatioLimit', 'seedRatioMode', 'downloadDir' ]
+    'recheckProgress', 'sizeWhenDone', 'status', 'trackerStats', 'desiredAvailable',
+    'uploadedEver', 'uploadRatio', 'seedRatioLimit', 'seedRatioMode', 'downloadDir', 'isFinished' ]
 
 Torrent.prototype =
 {
@@ -88,8 +88,9 @@ Torrent.prototype =
 		top_e.id = 'torrent_' + data.id;
 		top_e._torrent = this;
 		var element = $(top_e);
-                $(element).bind('dblclick', function(e) { transmission.toggleInspector(); });
+		$(element).bind('dblclick', function(e) { transmission.toggleInspector(); });
 		element._torrent = this;
+		element._id = this._id;
 		this._element = element;
 		this._controller = controller;
 		controller._rows.push( element );
@@ -243,7 +244,8 @@ Torrent.prototype =
 		switch( this.state() ) {
 			case Torrent._StatusSeeding:        return 'Seeding';
 			case Torrent._StatusDownloading:    return 'Downloading';
-			case Torrent._StatusPaused:         return 'Paused';
+			case Torrent._StatusPaused:         return this._isFinishedSeeding ? 'Seeding complete'
+                                                                               : 'Paused';
 			case Torrent._StatusChecking:       return 'Verifying local data';
 			case Torrent._StatusWaitingToCheck: return 'Waiting to verify';
 			default:                            return 'error';
@@ -391,6 +393,8 @@ Torrent.prototype =
 		this._state                   = data.status;
 		this._download_dir            = data.downloadDir;
 		this._metadataPercentComplete = data.metadataPercentComplete;
+		this._isFinishedSeeding       = data.isFinished;
+		this._desiredAvailable        = data.desiredAvailable;
 
 		if (data.fileStats)
 			this.refreshFileModel( data );
@@ -435,7 +439,7 @@ Torrent.prototype =
 				break;
 
 			case Torrent._StatusDownloading:
-				// 'Downloading from 36 of 40 peers - DL: 60.2 KB/s UL: 4.3 KB/s'
+				// 'Downloading from 36 of 40 peers - DL: 60.2 KiB/s UL: 4.3 KiB/s'
 				c = 'Downloading from ';
 				c += this.peersSendingToUs();
 				c += ' of ';
@@ -448,7 +452,7 @@ Torrent.prototype =
 				break;
 
 			case Torrent._StatusSeeding:
-				// 'Seeding to 13 of 22 peers - UL: 36.2 KB/s'
+				// 'Seeding to 13 of 22 peers - UL: 36.2 KiB/s'
 				c = 'Seeding to ';
 				c += this.peersGettingFromUs();
 				c += ' of ';
@@ -516,7 +520,7 @@ Torrent.prototype =
 			}
 			
 			// Create the 'progress details' label
-			// Eg: '101 MB of 631 MB (16.02%) - 2 hr remaining'
+			// Eg: '101 MiB of 631 MiB (16.02%) - 2 hr remaining'
 			c = Math.formatBytes( this._sizeWhenDone - this._leftUntilDone );
 			c += ' of ';
 			c += Math.formatBytes( this._sizeWhenDone );
@@ -545,8 +549,19 @@ Torrent.prototype =
 		}
 		else
 		{
+			var eta = '';
+
+			if( this.isActive( ) && this.seedRatioLimit( ) > 0 )
+			{
+				eta = ' - ';
+				if (this._eta < 0 || this._eta >= Torrent._InfiniteTimeRemaining )
+					eta += 'remaining time unknown';
+				else
+					eta += Math.formatSeconds(this._eta) + ' remaining';
+			}
+
 			// Create the 'progress details' label
-			// Eg: '698.05 MB, uploaded 8.59 GB (Ratio: 12.3)'
+			// Eg: '698.05 MiB, uploaded 8.59 GiB (Ratio: 12.3)'
 			c = Math.formatBytes( this._size );
 			c += ', uploaded ';
 			c += Math.formatBytes( this._upload_total );
@@ -558,6 +573,7 @@ Torrent.prototype =
 			else
 				c += '0';
 			c += ')';
+			c += eta;
 			progress_details = c;
 
 			var status = this.isActive() ? 'complete' : 'complete_stopped';
